@@ -23,6 +23,8 @@ using System;
 using System.IO;
 using System.Runtime.InteropServices;
 
+using Mono.Posix;
+
 using ICSharpCode.SharpZipLib.Checksums;
 using ICSharpCode.SharpZipLib.Zip;
 using ICSharpCode.SharpZipLib.GZip;
@@ -183,7 +185,9 @@ namespace FSpot {
 					progress_dialog.Message = Mono.Posix.Catalog.GetString ("Error While Transferring");
 				}
 
-				GnomeUtil.UrlShow (null, target.ToString ());
+				if (open) {
+					GnomeUtil.UrlShow (null, target.ToString ());
+				}
 			} catch (System.Exception e) {
 				progress_dialog.Message = e.ToString ();
 				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error Transferring");
@@ -282,6 +286,7 @@ namespace FSpot {
 		protected bool scale;
 		protected int size;
 		protected string description;
+		protected string language;
 		protected System.Uri destination;
 
 		protected ScaleRequest [] requests;
@@ -331,10 +336,10 @@ namespace FSpot {
 				ProcessImage (i);
 			}
 		}
-		
+
 		protected virtual string ImageName (int image_num)
 		{
-			return System.IO.Path.GetFileName (collection [image_num].DefaultVersionUri.LocalPath); 
+			return System.IO.Path.GetFileName(FileImportBackend.UniqueName(gallery_path, System.IO.Path.GetFileName (collection [image_num].DefaultVersionUri.LocalPath))); 
 		}
 
 		public void ProcessImage (int image_num)
@@ -374,14 +379,11 @@ namespace FSpot {
 				MakeDir (SubdirPath (req.Name));
 				path = SubdirPath (req.Name, ImageName (image_num));
 
-
-				
 				if (req.CopyExif && data != null) {
 					PixbufUtils.SaveJpeg (scaled, path, 90, data);
 				} else 
 					scaled.Savev (path, "jpeg", pixbuf_keys, pixbuf_values);
 				
-
 				if (img != null)
 					img.Dispose ();
 			}
@@ -428,6 +430,14 @@ namespace FSpot {
 				description = value;
 			}
 		}
+
+		public string Language {
+			get {
+				if (language == null)
+					language=GetLanguage();
+				return language;
+			}
+		}
 		
 		public Uri Destination {
 			get {
@@ -444,6 +454,24 @@ namespace FSpot {
 			requests [0].Width = size;
 			requests [0].Height = size;
 		}
+
+		private string GetLanguage()
+		{
+			string language;
+ 
+			if ((language = Environment.GetEnvironmentVariable ("LC_ALL")) == null)
+				if ((language = Environment.GetEnvironmentVariable ("LC_MESSAGES")) == null)
+					if ((language = Environment.GetEnvironmentVariable ("LANG")) == null)
+						language = "en";
+ 
+			if (language.IndexOf('.') >= 0)
+				language = language.Substring(0,language.IndexOf('.'));
+			if (language.IndexOf('@') >= 0)
+				language = language.Substring(0,language.IndexOf('@'));
+			language = language.Replace('_','-');
+ 
+			return language;
+		}
 	}
 
 	class OriginalGallery : FolderGallery
@@ -567,6 +595,9 @@ namespace FSpot {
 		string stylesheet = "f-spot-simple.css";
 		string altstylesheet = "f-spot-simple-white.css";
 		string javascript = "f-spot.js";
+
+        static string light = Mono.Posix.Catalog.GetString("Light");
+        static string dark = Mono.Posix.Catalog.GetString("Dark");
 		
 		public HtmlGallery (IBrowsableCollection selection, string path, string name) : base (selection, path, name) 
 		{ 
@@ -575,6 +606,11 @@ namespace FSpot {
 							 new ScaleRequest ("thumbs", 120, 90, false) };
 		}
 		
+		protected override string ImageName (int photo_index)
+		{
+			return String.Format ("img-{0}.jpg", photo_index + 1);
+		}
+
 		public override void Generate ()
 		{
 			if (collection.Count == 0)
@@ -676,6 +712,8 @@ namespace FSpot {
 			
 			//writer.Write ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
 			writer.WriteLine ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");
+			writer.AddAttribute ("xmlns", "http://www.w3.org/1999/xhtml");
+			writer.AddAttribute ("xml:lang", this.Language);
 			writer.RenderBeginTag ("html");
 			
 			WriteHeader (writer);
@@ -738,18 +776,18 @@ namespace FSpot {
 			writer.RenderBeginTag("ul");
 			writer.RenderBeginTag("li");
 			writer.AddAttribute ("href", "#");
-			writer.AddAttribute ("title", "Dark");
-			writer.AddAttribute ("onclick", "setActiveStyleSheet('Dark')");
+			writer.AddAttribute ("title", dark);
+			writer.AddAttribute ("onclick", "setActiveStyleSheet('" + dark + "')");
 			writer.RenderBeginTag("a");
-			writer.Write ("Dark");
+			writer.Write (dark);
 			writer.RenderEndTag (); //a
 			writer.RenderEndTag (); //li
 			writer.RenderBeginTag("li");
 			writer.AddAttribute ("href", "#");
-			writer.AddAttribute ("title", "Light");
-			writer.AddAttribute ("onclick", "setActiveStyleSheet('Light')");
+			writer.AddAttribute ("title", light);
+			writer.AddAttribute ("onclick", "setActiveStyleSheet('" + light + "')");
 			writer.RenderBeginTag("a");
-			writer.Write ("Light");
+			writer.Write (light);
 			writer.RenderEndTag (); //a
 			writer.RenderEndTag (); //li
 			writer.RenderEndTag (); //ul
@@ -757,8 +795,8 @@ namespace FSpot {
 			writer.RenderBeginTag ("div");
 			writer.Write ("<span class=\"style_toggle\">"); 
 			writer.Write ("<a href=\"javascript:toggle_stylebox()\">");
-			writer.Write ("<span id=\"showlink\">Show Styles</span><span id=\"hidelink\" ");
-			writer.Write ("style=\"display:none;\">Hide Styles</span></a></span>\n");
+			writer.Write ("<span id=\"showlink\">" + Catalog.GetString ("Show Styles") + "</span><span id=\"hidelink\" ");
+			writer.Write ("style=\"display:none;\">" + Catalog.GetString ("Hide Styles") + "</span></a></span>\n");
 			writer.RenderEndTag (); //div toggle
 			writer.RenderEndTag (); //div styleboxcontainer
 			writer.RenderEndTag (); //container1	
@@ -788,18 +826,21 @@ namespace FSpot {
 		public void WriteHeader (System.Web.UI.HtmlTextWriter writer)
 		{
 			writer.RenderBeginTag ("head");
+			/* It seems HtmlTextWriter always uses UTF-8, unless told otherwise */
+			writer.Write ("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />");
+			writer.WriteLine ();
 			writer.RenderBeginTag ("title");
 			writer.Write (gallery_name);
 			writer.RenderEndTag ();
 
 			writer.Write ("<link type=\"text/css\" rel=\"stylesheet\" href=\"");
 			writer.Write (String.Format ("{0}", "style/" + stylesheet));
-			writer.Write ("\" title=\"Dark\" media=\"screen\" />\n");
+			writer.Write ("\" title=\"" + dark + "\" media=\"screen\" />\n");
 
 			writer.Write ("<link type=\"text/css\" rel=\"prefetch ") ;
 			writer.Write ("alternate stylesheet\" href=\"");
 			writer.Write (String.Format ("{0}", "style/" + altstylesheet));
-			writer.Write ("\" title=\"Light\" media=\"screen\" />\n");
+			writer.Write ("\" title=\"" + light + "\" media=\"screen\" />\n");
 
 			writer.Write ("<script src=\"script/" + javascript + "\"");
 			writer.Write (" type=\"text/javascript\"></script>\n");
@@ -831,6 +872,8 @@ namespace FSpot {
 
 			//writer.Write ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
 			writer.WriteLine ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");
+			writer.AddAttribute ("xmlns", "http://www.w3.org/1999/xhtml");
+			writer.AddAttribute ("xml:lang", this.Language);
 			writer.RenderBeginTag ("html");
 			WriteHeader (writer);
 			
@@ -906,18 +949,18 @@ namespace FSpot {
 			writer.RenderBeginTag("ul");
 			writer.RenderBeginTag("li");
 			writer.AddAttribute ("href", "#");
-			writer.AddAttribute ("title", "Dark");
-			writer.AddAttribute ("onclick", "setActiveStyleSheet('Dark')");
+			writer.AddAttribute ("title", dark);
+			writer.AddAttribute ("onclick", "setActiveStyleSheet('" + dark + "')");
 			writer.RenderBeginTag("a");
-			writer.Write ("Dark");
+			writer.Write (dark);
 			writer.RenderEndTag (); //a
 			writer.RenderEndTag (); //li
 			writer.RenderBeginTag("li");
 			writer.AddAttribute ("href", "#");
-			writer.AddAttribute ("title", "Light");
-			writer.AddAttribute ("onclick", "setActiveStyleSheet('Light')");
+			writer.AddAttribute ("title", light);
+			writer.AddAttribute ("onclick", "setActiveStyleSheet('" + light + "')");
 			writer.RenderBeginTag("a");
-			writer.Write ("Light");
+			writer.Write (light);
 			writer.RenderEndTag (); //a
 			writer.RenderEndTag (); //li
 			writer.RenderEndTag (); //ul
@@ -925,8 +968,8 @@ namespace FSpot {
 			writer.RenderBeginTag ("div");
 			writer.Write ("<span class=\"style_toggle\">"); 
 			writer.Write ("<a href=\"javascript:toggle_stylebox()\">");
-			writer.Write ("<span id=\"showlink\">Show Styles</span><span id=\"hidelink\" ");
-			writer.Write ("style=\"display:none;\">Hide Styles</span></a></span>\n");
+			writer.Write ("<span id=\"showlink\">" + Mono.Posix.Catalog.GetString("Show Styles") + "</span><span id=\"hidelink\" ");
+			writer.Write ("style=\"display:none;\">" + Mono.Posix.Catalog.GetString("Hide Styles") + "</span></a></span>\n");
 			writer.RenderEndTag (); //div toggle
 			writer.RenderEndTag (); //div styleboxcontainer
 			writer.RenderEndTag (); //container1
