@@ -1,32 +1,88 @@
+using System;
+using System.Net;
+using System.IO;
+using System.Text;
+using System.Collections;
+using System.Collections.Specialized;
+using System.Web;
+using Mono.Unix;
+
+using GalleryRemote;
+
 namespace FSpot {
 	public class GalleryAccount {
-		public GalleryAccount (string name, string url, string username, string password)
+		public GalleryAccount (string name, string url, string username, string password) : this (name, url, username, password, GalleryVersion.VersionUnknown) {}
+		public GalleryAccount (string name, string url, string username, string password, GalleryVersion version)
 		{
 			this.name = name;
 			this.username = username;
 			this.password = password;
 			this.Url = url;
+
+			if (version != GalleryVersion.VersionUnknown) {
+				this.version = version;
+			} else {
+				this.version = Gallery.DetectGalleryVersion(Url);
+			}
 		}
-		
-		public GalleryRemote.Gallery Connect ()
+
+		public Gallery Connect ()
 		{
-			GalleryRemote.Gallery gal = null;
-			gal = new GalleryRemote.Gallery (url);
-			gal.Login (username, password);
-			gallery = gal;
-			connected = true;
+			//System.Console.WriteLine ("GalleryAccount.Connect()");
+			
+			Gallery gal = null;
+
+			if (version == GalleryVersion.VersionUnknown)
+				this.version = Gallery.DetectGalleryVersion(Url);
+
+			if (version == GalleryVersion.Version1) {
+				gal = new Gallery1 (url, url);
+			} else if (version == GalleryVersion.Version2) {
+				gal = new Gallery2 (url, url);
+			} else {
+				throw new Exception ("Cannot connect to a Gallery for which the version is unknown");
+			}
+
+			System.Console.WriteLine ("Gallery created: " + gal);
+
+			if (gal.Login (username, password)) {
+				//System.Console.WriteLine ("Login successful");
+				gallery = gal;
+				connected = true;
+			}
+
 			return gallery;
 		}
 
+		GalleryVersion version;
+		public GalleryVersion Version{
+			get {
+				return version;
+			}
+		}
+
 		private bool connected;
 		public bool Connected {
 			get {
-				return connected;
+				bool retVal = false;
+				if(gallery != null) {
+					retVal = gallery.IsConnected ();
+				}
+				if (connected != retVal) {
+					System.Console.WriteLine ("Connected and retVal for IsConnected() don't agree");
+				}
+				return retVal;
 			}
 		}
 
-		GalleryRemote.Gallery gallery;
-		public GalleryRemote.Gallery Gallery {
+		public void MarkChanged ()
+		{
+			connected = false;
+			gallery = null;
+		}
+
+		Gallery gallery;
+		public Gallery Gallery {
 			get {
 				return gallery;
 			}
@@ -48,9 +104,10 @@ namespace FSpot {
 				return url;
 			}
 			set {
-				url = value;
-				if (url != null && !url.EndsWith ("/gallery_remote2.php"))
-					url = url + "/gallery_remote2.php";
+				if (url != value) {
+					url = value;
+					MarkChanged ();
+				}
 			}
 		}
 
@@ -60,7 +117,10 @@ namespace FSpot {
 				return username;
 			}
 			set {
-				username = value;
+				if (username != value) {
+					username = value;
+					MarkChanged ();
+				}
 			}
 		}
 
@@ -70,30 +130,188 @@ namespace FSpot {
 				return password;
 			}
 			set {
-				password = value;
+				if (password != value) {
+					password = value;
+					MarkChanged ();
+				}
 			}
 		}
 	}
 
 	
+	public class GalleryAccountManager 
+	{
+		private static GalleryAccountManager instance;
+		string xml_path;
+		ArrayList accounts;
 
+		public delegate void AccountListChangedHandler (GalleryAccountManager manager, GalleryAccount changed_account);
+		public event AccountListChangedHandler AccountListChanged;
+		
+		public static GalleryAccountManager GetInstance ()
+		{
+			if (instance == null) {
+				instance = new GalleryAccountManager ();
+			}
+
+			return instance;
+		}
+
+		private GalleryAccountManager ()
+		{
+			// FIXME this xml file path should be be retrieved from a central location not hard coded there
+			this.xml_path = System.IO.Path.Combine (FSpot.Global.BaseDirectory, "Accounts.xml");
+			
+			accounts = new ArrayList ();
+			ReadAccounts ();
+		}	
+
+		public void MarkChanged ()
+		{
+			MarkChanged (true, null);
+		}
+
+		public void MarkChanged (bool write, GalleryAccount changed_account)
+		{
+			if (write)
+				WriteAccounts ();
+
+			if (AccountListChanged != null)
+				AccountListChanged (this, changed_account);
+		}
+
+		public ArrayList GetAccounts ()
+		{
+			return accounts;
+		}
+
+		public void AddAccount (GalleryAccount account)
+		{
+			AddAccount (account, true);
+		}
+
+		public void AddAccount (GalleryAccount account, bool write)
+		{
+			accounts.Add (account);
+			MarkChanged (write, account);
+		}
+
+		public void RemoveAccount (GalleryAccount account)
+		{
+			accounts.Remove (account);
+			MarkChanged ();
+		}
+
+		public void WriteAccounts ()
+		{
+			System.Xml.XmlTextWriter writer = new System.Xml.XmlTextWriter (xml_path, System.Text.Encoding.Default);
+			
+			writer.Formatting = System.Xml.Formatting.Indented;
+			writer.Indentation = 2;
+			writer.IndentChar = ' ';
+
+			writer.WriteStartDocument (true);
+			
+			writer.WriteStartElement ("GalleryRemote");
+			foreach (GalleryAccount account in accounts) {
+				writer.WriteStartElement ("Account");
+				writer.WriteElementString ("Name", account.Name);
+				
+				writer.WriteElementString ("Url", account.Url);
+				writer.WriteElementString ("Username", account.Username);
+				writer.WriteElementString ("Password", account.Password);
+				writer.WriteElementString ("Version", account.Version.ToString());
+				writer.WriteEndElement (); //Account
+			}
+			writer.WriteEndElement ();
+			writer.WriteEndDocument ();
+			writer.Close ();
+		}
+
+		private GalleryAccount ParseAccount (System.Xml.XmlNode node) 
+		{
+			if (node.Name != "Account")
+				
+				return null;
+
+			string name = null;
+			string url = null;
+			string username = null;
+			string password = null;
+			GalleryVersion version = GalleryVersion.VersionUnknown;
+
+			foreach (System.Xml.XmlNode child in node.ChildNodes) {
+				if (child.Name == "Name") {
+					name = child.ChildNodes [0].Value;
+				
+				} else if (child.Name == "Url") {
+					url = child.ChildNodes [0].Value;
+				} else if (child.Name == "Password") {
+					password = child.ChildNodes [0].Value;
+				} else if (child.Name == "Username") {
+					username = child.ChildNodes [0].Value;
+				} else if (child.Name == "Version") {
+					string versionString = child.ChildNodes [0].Value;
+					if (versionString == "Version1") 
+						version = GalleryVersion.Version1;
+					else if (versionString == "Version2")
+						version = GalleryVersion.Version2;
+					else
+						Console.WriteLine ("Unexpected versions string: " + versionString);
+				}
+			}
+			return new GalleryAccount (name, url, username, password, version);
+		}
+
+		private void ReadAccounts ()
+		{
+
+			if (! File.Exists (xml_path)) {
+				MarkChanged ();
+				return;
+			}
+
+			try {
+				string query = "//GalleryRemote/Account";
+				System.Xml.XmlDocument doc = new System.Xml.XmlDocument ();
+				
+				//System.Console.WriteLine ("xml_path: " + xml_path);
+				doc.Load (xml_path);
+				System.Xml.XmlNodeList nodes = doc.SelectNodes (query);
+				
+				//System.Console.WriteLine ("selected {0} nodes match {1}", nodes.Count, query);
+				foreach (System.Xml.XmlNode node in nodes) {
+					GalleryAccount account = ParseAccount (node);
+					if (account != null)
+						AddAccount (account, false);
+				
+				}
+			} catch (System.Exception e) {
+				// FIXME do something
+				System.Console.WriteLine ("Exception loading gallery accounts");
+				System.Console.WriteLine (e);
+			}
+
+			MarkChanged ();
+		}
+	}
+	
 	public class AccountDialog : GladeDialog {
-		public AccountDialog (GalleryExport export) : this (export, null) { 
+		public AccountDialog (Gtk.Window parent) : this (parent, null, false) { 
 			Dialog.Response += HandleAddResponse;
 			add_button.Sensitive = false;
-			status_area.Visible = false;
 		}
 		
-		public AccountDialog (GalleryExport export, GalleryAccount account) :  base ("gallery_add_dialog")
+		public AccountDialog (Gtk.Window parent, GalleryAccount account, bool show_error) :  base ("gallery_add_dialog")
 		{
-			this.export = export;
-
 			this.Dialog.Modal = false;
-			this.Dialog.TransientFor = export.Dialog;
-			this.Dialog.Show ();
+			this.Dialog.TransientFor = parent;
+			this.Dialog.DefaultResponse = Gtk.ResponseType.Ok;
 			
 			this.account = account;
 
+			status_area.Visible = show_error;
+
 			if (account != null) {
 				gallery_entry.Text = account.Name;
 				url_entry.Text = account.Url;
@@ -101,7 +319,12 @@ namespace FSpot {
 				username_entry.Text = account.Username;
 				add_button.Label = Gtk.Stock.Ok;
 				Dialog.Response += HandleEditResponse;
-			} 
+			}
+
+			if (remove_button != null)
+				remove_button.Visible = account != null;
+
+			this.Dialog.Show ();
 
 			gallery_entry.Changed += HandleChanged;
 			url_entry.Changed += HandleChanged;
@@ -121,19 +344,37 @@ namespace FSpot {
 				add_button.Sensitive = false;
 			else
 				add_button.Sensitive = true;
+
 		}
 		
 		[GLib.ConnectBefore]
 		protected void HandleAddResponse (object sender, Gtk.ResponseArgs args)
 		{
-
-
 			if (args.ResponseId == Gtk.ResponseType.Ok) {
+				try {
+					Uri uri = new Uri (url);
+					if (uri.Scheme != Uri.UriSchemeHttp &&
+					    uri.Scheme != Uri.UriSchemeHttps)
+						throw new System.UriFormatException ();
+
+				} catch (System.UriFormatException e) {
+					HigMessageDialog md = 
+						new HigMessageDialog (Dialog, 
+								      Gtk.DialogFlags.Modal |
+								      Gtk.DialogFlags.DestroyWithParent,
+								      Gtk.MessageType.Error, Gtk.ButtonsType.Ok,
+								      Catalog.GetString ("Invalid URL"),
+								      Catalog.GetString ("The gallery URL entry does not appear to be a valid URL"));
+					md.Run ();
+					md.Destroy ();
+					return;
+				}
+
 				GalleryAccount account = new GalleryAccount (name, 
 									     url, 
 									     username,
 									     password);
-				export.AddAccount (account);
+				GalleryAccountManager.GetInstance ().AddAccount (account);
 			}
 			Dialog.Destroy ();
 		}
@@ -145,11 +386,14 @@ namespace FSpot {
 				account.Url = url;
 				account.Username = username;
 				account.Password = password;
+				GalleryAccountManager.GetInstance ().MarkChanged (true, account);
+			} else if (args.ResponseId == Gtk.ResponseType.Reject) {
+				// NOTE we are using Reject to signal the remove action.
+				GalleryAccountManager.GetInstance ().RemoveAccount (account);
 			}
+			Dialog.Destroy ();				
 		}
 
-		GalleryExport export;
-
 		private GalleryAccount account;
 		private string name;
 		private string url;
@@ -163,6 +407,7 @@ namespace FSpot {
 		[Glade.Widget] Gtk.Entry username_entry;
 
 		[Glade.Widget] Gtk.Button add_button;
+		[Glade.Widget] Gtk.Button remove_button;
 		[Glade.Widget] Gtk.Button cancel_button;
 
 		[Glade.Widget] Gtk.HBox status_area;
@@ -175,58 +420,95 @@ namespace FSpot {
 		[Glade.Widget] Gtk.Entry description_entry;
 		[Glade.Widget] Gtk.Entry title_entry;
 		
-		GalleryRemote.Gallery gallery;
-		GalleryRemote.Album parent;
+		[Glade.Widget] Gtk.Button add_button;
+		[Glade.Widget] Gtk.Button cancel_button;
 
-		public GalleryAddAlbum (GalleryRemote.Gallery gallery) : base ("gallery_add_album_dialog")
+		private GalleryExport export;
+		private Gallery gallery;
+		private string parent;
+		private string name;
+		private string description;
+		private string title;
+
+		public GalleryAddAlbum (GalleryExport export, Gallery gallery) : base ("gallery_add_album_dialog")
 		{
+			this.export = export;
+			this.gallery = gallery;	
+			PopulateAlbums ();
 			
-		}
-#if false
-		public GalleryRemote.Album Parent {
-			get {
-				return Parent;
-			}
-		}
-
-		public bool Run {
+			Dialog.Response += HandleAddResponse;
 
+			name_entry.Changed += HandleChanged;
+			description_entry.Changed += HandleChanged;
+			title_entry.Changed += HandleChanged;
+			HandleChanged (null, null);
 		}
 		
-		private PopulateAlbums ()
+		private void PopulateAlbums ()
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
-			foreach (GalleryRemote.Album album in albums) {
+			if (gallery.Version == GalleryVersion.Version1) {
+				Gtk.MenuItem top_item = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("(TopLevel)"));
+				menu.Append (top_item);
+			}
+			
+			foreach (Album album in gallery.Albums) {
 				System.Text.StringBuilder label_builder = new System.Text.StringBuilder ();
 				
-				for (GalleryRemote.Album parent = album.Parent ();
-				     parent != null;
-				     parent = parent.Parent ()) {
+				for (int i=0; i < album.Parents.Count; i++) {
 					label_builder.Append ("  ");
-					//Console.WriteLine ("looping");
 				}
 				label_builder.Append (album.Title);
 				
 				Gtk.MenuItem item = new Gtk.MenuItem (label_builder.ToString ());
+				((Gtk.Label)item.Child).UseUnderline = false;
 				menu.Append (item);
 				
-				GalleryRemote.AlbumPermission add_permission = album.Perms & GalleryRemote.AlbumPermission.Add;
+				AlbumPermission create_sub = album.Perms & AlbumPermission.CreateSubAlbum;
 				
-				if (add_permission == 0)
+				if (create_sub == 0)
 					item.Sensitive = false;
 			}
+
+			album_optionmenu.Sensitive = true;
+			menu.ShowAll ();
+			album_optionmenu.Menu = menu;
+		}
+		
+		private void HandleChanged (object sender, EventArgs args)
+		{
+			if (gallery.Albums.Count == 0 || album_optionmenu.History <= 0) {
+				parent = "";
+			} else {
+				parent = ((Album) gallery.Albums [album_optionmenu.History-1]).Name;
+			}
+			name = name_entry.Text;
+			description = description_entry.Text;
+			title = title_entry.Text;
+
+			if (name == "" || title == "")
+				add_button.Sensitive = false;
+			else
+				add_button.Sensitive = true;
+		}
+		
+		[GLib.ConnectBefore]
+		protected void HandleAddResponse (object sender, Gtk.ResponseArgs args)
+		{
+			if (args.ResponseId == Gtk.ResponseType.Ok) {
+				gallery.NewAlbum (parent, name, title, description);
+				export.HandleAlbumAdded (title);
+			}
+			Dialog.Destroy ();
 		}
-#endif
 	}
+
 	
 	public class GalleryExport : GladeDialog {
 		public GalleryExport (IBrowsableCollection selection) : base ("gallery_export_dialog")
 		{
 			this.photos = (Photo []) selection.Items;
-
-			// FIXME this xml file path should be be retrieved from a central location not hard coded there
-			this.xml_path = System.IO.Path.Combine (FSpot.Global.BaseDirectory, "Accounts.xml");
-			
+			album_button.Sensitive = false;
 			IconView view = new IconView (selection);
 			view.DisplayDates = false;
 			view.DisplayTags = false;
@@ -238,8 +520,14 @@ namespace FSpot {
 			view.Show ();
 			Dialog.Show ();
 
-			LoadAccounts ();
 
+			GalleryAccountManager manager = GalleryAccountManager.GetInstance ();
+			manager.AccountListChanged += PopulateGalleryOptionMenu;
+			PopulateGalleryOptionMenu (manager, null);
+
+			if (edit_button != null)
+				edit_button.Clicked += HandleEditGallery;
+			
 			rh = new Gtk.ResponseHandler (HandleResponse);
 			Dialog.Response += HandleResponse;
 			connect = true;
@@ -264,9 +552,9 @@ namespace FSpot {
 		int photo_index;
 		FSpot.ThreadProgressDialog progress_dialog;
 		
-		System.Collections.ArrayList accounts;
+		ArrayList accounts;
 		private GalleryAccount account;
-		private GalleryRemote.Album album;
+		private Album album;
 
 		private string xml_path;
 
@@ -289,6 +577,7 @@ namespace FSpot {
 
 		[Glade.Widget] Gtk.Button album_button;
 		[Glade.Widget] Gtk.Button add_button;
+		[Glade.Widget] Gtk.Button edit_button;
 		
 		[Glade.Widget] Gtk.Button ok_button;
 		[Glade.Widget] Gtk.Button cancel_button;
@@ -297,73 +586,6 @@ namespace FSpot {
 
 		System.Threading.Thread command_thread;
 		
-		public void WriteAccounts ()
-		{
-			System.Xml.XmlTextWriter writer = new System.Xml.XmlTextWriter (xml_path, System.Text.Encoding.Default);
-			writer.Formatting = System.Xml.Formatting.Indented;
-			writer.Indentation = 2;
-			writer.IndentChar = ' ';
-
-			writer.WriteStartDocument (true);
-			
-			writer.WriteStartElement ("GalleryRemote");
-			foreach (GalleryAccount account in accounts) {
-				writer.WriteStartElement ("Account");
-				writer.WriteElementString ("Name", account.Name);
-				writer.WriteElementString ("Url", account.Url);
-				writer.WriteElementString ("Username", account.Username);
-				writer.WriteElementString ("Password", account.Password);
-				writer.WriteEndElement (); //Account
-			}
-			writer.WriteEndElement ();
-			writer.WriteEndDocument ();
-			writer.Close ();
-		}
-
-		private GalleryAccount ParseAccount (System.Xml.XmlNode node) 
-		{
-			if (node.Name != "Account")
-				return null;
-
-			string name = null;
-			string url = null;
-			string username = null;
-			string password = null;
-
-			foreach (System.Xml.XmlNode child in node.ChildNodes) {
-				if (child.Name == "Name") {
-					name = child.ChildNodes [0].Value;
-				} else if (child.Name == "Url") {
-					url = child.ChildNodes [0].Value;
-				} else if (child.Name == "Password") {
-					password = child.ChildNodes [0].Value;
-				} else if (child.Name == "Username") {
-					username = child.ChildNodes [0].Value;
-				}
-			}
-			return new GalleryAccount (name, url, username, password);
-		}
-
-		private void ReadAccounts ()
-		{
-			try {
-				string query = "//GalleryRemote/Account";
-				System.Xml.XmlDocument doc = new System.Xml.XmlDocument ();
-				doc.Load (xml_path);
-				System.Xml.XmlNodeList nodes = doc.SelectNodes (query);
-				
-				System.Console.WriteLine ("selected {0} nodes match {1}", nodes.Count, query);
-				foreach (System.Xml.XmlNode node in nodes) {
-					GalleryAccount account = ParseAccount (node);
-					if (account != null)
-						AddAccount (account, false);
-				}
-			} catch (System.Exception e) {
-				// FIXME do something
-				PopulateGalleryOptionMenu ();
-				PopulateAlbumOptionMenu (null);
-			}
-		}
 
 		private void HandleResponse (object sender, Gtk.ResponseArgs args)
 		{
@@ -382,8 +604,8 @@ namespace FSpot {
 			meta = meta_check.Active;
 
 			if (account != null) { 
-				System.Console.WriteLine ("history = {0}", album_optionmenu.History);
-				album = (GalleryRemote.Album) account.Gallery.Albums [System.Math.Max (0, album_optionmenu.History)]; 
+				//System.Console.WriteLine ("history = {0}", album_optionmenu.History);
+				album = (Album) account.Gallery.Albums [Math.Max (0, album_optionmenu.History)]; 
 				photo_index = 0;
 				
 				Dialog.Destroy ();
@@ -408,7 +630,7 @@ namespace FSpot {
 			progress_dialog.Fraction = (photo_index - 1.0 + item.Value) / (double) photos.Length;
 		}
 
-		public void HandleSizeActive (object sender, System.EventArgs args)
+		public void HandleSizeActive (object sender, EventArgs args)
 		{
 			size_spin.Sensitive = scale_check.Active;
 		}
@@ -433,7 +655,7 @@ namespace FSpot {
 					
 					if (scale) {
 						/* FIXME this is sick */
-						string orig = photo.DefaultVersionPath;
+						string orig = photo.DefaultVersionUri.LocalPath;
 						string path = PixbufUtils.Resize (orig, size, true);
 						string final = path + System.IO.Path.GetExtension (orig);
 						System.IO.File.Move (path, final);
@@ -449,8 +671,10 @@ namespace FSpot {
 				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Upload Complete");
 				progress_dialog.ButtonLabel = Gtk.Stock.Ok;
 			} catch (System.Exception e) {
-				progress_dialog.Message = e.ToString ();
-				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error Uploading To Gallery");
+				progress_dialog.Message = String.Format (Mono.Posix.Catalog.GetString ("Error Uploading To Gallery: {0}"),
+									 e.Message);
+				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error");
+				System.Console.WriteLine (e);
 			}
 			
 			if (browser) {
@@ -458,65 +682,70 @@ namespace FSpot {
 			}
 		}
 		
-		private void LoadAccounts ()
-		{ 
-			accounts = new System.Collections.ArrayList ();
-			ReadAccounts ();
-		}
-
-		public void AddAccount (GalleryAccount account)
-		{
-			AddAccount (account, true);
-		}
-
-		public void AddAccount (GalleryAccount account, bool write)
-		{
-			accounts.Add (account);
-			PopulateGalleryOptionMenu ();
-			if (write)
-				WriteAccounts ();
-		}
-
-		private void PopulateGalleryOptionMenu ()
+		private void PopulateGalleryOptionMenu (GalleryAccountManager manager, GalleryAccount changed_account)
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
-			
+			this.account = changed_account;
+			int pos = -1;
+
+			accounts = manager.GetAccounts ();
 			if (accounts == null || accounts.Count == 0) {
 				Gtk.MenuItem item = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("(No Gallery)"));
 				menu.Append (item);
 				gallery_optionmenu.Sensitive = false;
+				edit_button.Sensitive = false;
 			} else {
+				int i = 0;
 				foreach (GalleryAccount account in accounts) {
+					if (account == changed_account)
+						pos = i;
+					
 					Gtk.MenuItem item = new Gtk.MenuItem (account.Name);
 					menu.Append (item);		
+					i++;
 				}
 				gallery_optionmenu.Sensitive = true;
+				edit_button.Sensitive = true;
 			}
 
 			menu.ShowAll ();
 			gallery_optionmenu.Menu = menu;
+			gallery_optionmenu.SetHistory ((uint)pos);
 		}
 
 		private void Connect ()
+		{
+			Connect (null);
+		}
+
+		private void Connect (GalleryAccount selected)
 		{
 			try {
 				if (accounts.Count != 0 && connect) {
-					account = (GalleryAccount) accounts [gallery_optionmenu.History];
+					if (selected == null)
+						account = (GalleryAccount) accounts [gallery_optionmenu.History];
+					else
+						account = selected;
+
 					if (!account.Connected)
 						account.Connect ();
 					
 					PopulateAlbumOptionMenu (account.Gallery);
+					album_button.Sensitive = true;
 				}
 			} catch (System.Exception ex) {
+				if (selected != null)
+					account = selected;
+
+				System.Console.WriteLine ("{0}",ex);
 				PopulateAlbumOptionMenu (account.Gallery);
+				album_button.Sensitive = false;
 				
-				AccountDialog dialog = new AccountDialog (this, account);
+				AccountDialog dialog = new AccountDialog (this.Dialog, account, true);
 				Gtk.ResponseType response = (Gtk.ResponseType) dialog.Dialog.Run ();
 
-				dialog.Dialog.Destroy ();				
 				if (response == Gtk.ResponseType.Ok) {
-					WriteAccounts ();
-					Connect ();
+					Connect (account);					
 				}
 			} 
 		}
@@ -526,10 +755,24 @@ namespace FSpot {
 			Connect ();
 		}
 
-		private void PopulateAlbumOptionMenu (GalleryRemote.Gallery gallery)
+		public void HandleAlbumAdded (string title) {
+			GalleryAccount account = (GalleryAccount) accounts [gallery_optionmenu.History];
+			PopulateAlbumOptionMenu (account.Gallery);
+				
+			// make the newly created album selected
+			ArrayList albums = account.Gallery.Albums;
+			for (int i=0; i < albums.Count; i++) {
+				if (((Album)albums[i]).Title == title) {
+					album_optionmenu.SetHistory((uint)i);
+				}
+			}
+		}
+
+		private void PopulateAlbumOptionMenu (Gallery gallery)
 		{
 			System.Collections.ArrayList albums = null;
 			if (gallery != null) {
+				//gallery.FetchAlbumsPrune ();
 				gallery.FetchAlbums ();
 				albums = gallery.Albums;
 			}
@@ -548,22 +791,23 @@ namespace FSpot {
 				ok_button.Sensitive = false;
 				album_optionmenu.Sensitive = false;
 				album_button.Sensitive = false;
+
+				if (disconnected) 
+					album_button.Sensitive = false;
 			} else {
-				foreach (GalleryRemote.Album album in albums) {
+				foreach (Album album in albums) {
 					System.Text.StringBuilder label_builder = new System.Text.StringBuilder ();
 					
-					for (GalleryRemote.Album parent = album.Parent ();
-					     parent != null;
-					     parent = parent.Parent ()) {
+					for (int i=0; i < album.Parents.Count; i++) {
 						label_builder.Append ("  ");
-						//Console.WriteLine ("looping");
 					}
 					label_builder.Append (album.Title);
 
 					Gtk.MenuItem item = new Gtk.MenuItem (label_builder.ToString ());
+					((Gtk.Label)item.Child).UseUnderline = false;
 					menu.Append (item);
 			
-				        GalleryRemote.AlbumPermission add_permission = album.Perms & GalleryRemote.AlbumPermission.Add;
+				        AlbumPermission add_permission = album.Perms & AlbumPermission.Add;
 
 					if (add_permission == 0)
 						item.Sensitive = false;
@@ -580,12 +824,20 @@ namespace FSpot {
 		
 		public void HandleAddGallery (object sender, System.EventArgs args)
 		{
-			gallery_add = new AccountDialog (this);
+			gallery_add = new AccountDialog (this.Dialog);
+		}
+		
+		public void HandleEditGallery (object sender, System.EventArgs args)
+		{
+			gallery_add = new AccountDialog (this.Dialog, account, false);
 		}
 
 		public void HandleAddAlbum (object sender, System.EventArgs args)
 		{
-			album_add = new GalleryAddAlbum (account.Gallery);
+			if (account == null)
+				throw new Exception (Catalog.GetString ("No account selected"));
+				
+			album_add = new GalleryAddAlbum (this, account.Gallery);
 		}
 
 		void LoadPreference (string key)
@@ -595,7 +847,7 @@ namespace FSpot {
 			if (val == null)
 				return;
 			
-			//System.Console.WriteLine("Setting {0} to {1}", key, val);
+			//System.Console.WriteLine ("Setting {0} to {1}", key, val);
 
 			switch (key) {
 			case Preferences.EXPORT_GALLERY_SCALE:
