@@ -197,7 +197,7 @@ public class IconView : Gtk.Layout {
 		
 		CanFocus = true;
 
-		FSpot.Global.ModifyColors (this);
+		//FSpot.Global.ModifyColors (this);
 	}
 	
 	public IconView (FSpot.IBrowsableCollection collection) : this () 
@@ -736,12 +736,13 @@ public class IconView : Gtk.Layout {
 			entry.Data = thumbnail_num;
 
 		bool selected = selection.Contains (thumbnail_num);
-		StateType cell_state = selected ? (HasFocus ? StateType.Selected :StateType.Active) : StateType.Normal;
+		StateType cell_state = selected ? (HasFocus ? StateType.Selected : StateType.Active) : State;
 		
-		Style.PaintFlatBox (Style, BinWindow, cell_state, 
-				    ShadowType.Out, area, this, "IconView", 
-				    bounds.X, bounds.Y,
-				    bounds.Width - 1, bounds.Height - 1);
+		if (cell_state != State)
+			Style.PaintBox (Style, BinWindow, cell_state, 
+					ShadowType.Out, area, this, "IconView", 
+					bounds.X, bounds.Y,
+					bounds.Width - 1, bounds.Height - 1);
 		
 		Gdk.Rectangle focus = Gdk.Rectangle.Inflate (bounds, -3, -3);
 
@@ -1196,13 +1197,13 @@ public class IconView : Gtk.Layout {
 			System.Uri uri = collection [order].DefaultVersionUri;
 
 			if (result == null && !System.IO.File.Exists (FSpot.ThumbnailGenerator.ThumbnailPath (uri)))
-				FSpot.ThumbnailGenerator.Default.Request (uri.LocalPath, 0, 256, 256);
+				FSpot.ThumbnailGenerator.Default.Request (uri, 0, 256, 256);
 			
 			if (result == null)
 				return;
 			
 			if (!FSpot.PhotoLoader.ThumbnailIsValid (uri, result))
-				FSpot.ThumbnailGenerator.Default.Request (uri.LocalPath, 0, 256, 256);
+				FSpot.ThumbnailGenerator.Default.Request (uri, 0, 256, 256);
 		}
 			
 		if (result == null)
@@ -1282,6 +1283,32 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
+	private void SetColors () 
+	{
+		if (IsRealized) {
+			BinWindow.Background = Style.BaseColors [(int)State];
+			GdkWindow.Background = Style.BaseColors [(int)State];
+		}
+	}
+
+	protected override void OnRealized ()
+	{
+		base.OnRealized ();
+		SetColors ();
+	}
+
+	protected override void OnStateChanged (StateType previous)
+	{
+		base.OnStateChanged (previous);
+		SetColors ();
+	}
+	
+	protected override void OnStyleSet (Style previous)
+	{
+		base.OnStyleSet (previous);
+		SetColors ();
+	}
+
 	protected override void OnSizeAllocated (Gdk.Rectangle allocation)
 	{
 		scroll_value = (Vadjustment.Value)/ (Vadjustment.Upper);
