@@ -1,3 +1,4 @@
+#define USE_TIFF
 using System;
 using System.IO;
 using FSpot.Xmp;
@@ -14,19 +15,23 @@ namespace FSpot {
 		private JpegHeader header;
 		private FSpot.Tiff.Header exif_header;
 		
+		public JpegFile (Uri uri) : base (uri)
+		{
+			try {
+				System.Console.WriteLine ("approximate quality = {0}", Header.GuessQuality ());
+			} catch (Exception e) {
+				System.Console.WriteLine (e);
+			}
+		}
+		
 		public JpegFile (string path) : base (path) 
 		{
-#if false // TEST_METADATA
-			MetadataStore store = new MetadataStore ();
-			Select (store);
-			store.Dump ();
-#endif
 		}
 
 		public JpegHeader Header {
 			get {
 				if (header == null) {
-					using (System.IO.FileStream stream = System.IO.File.OpenRead (path)) {
+					using (Stream stream = this.Open ()) {
 						header = new JpegHeader (stream, true);
 					}
 				}
@@ -39,7 +44,6 @@ namespace FSpot {
 				if (exif_header == null) {
 					exif_header = Header.GetExifHeader ();
 				}
-				
 				return exif_header;
 			}
 		}
@@ -56,6 +60,19 @@ namespace FSpot {
 
 		public override string Description {
 			get {
+#if USE_TIFF
+				try {
+					SubdirectoryEntry sub = (SubdirectoryEntry) ExifHeader.Directory.Lookup (TagId.ExifIfdPointer);
+					if (sub != null) {
+						DirectoryEntry entry = sub.Directory [0].Lookup (TagId.UserComment);
+						if (entry != null)
+							return entry.ValueAsString [0];
+					}
+				} catch (System.Exception e) {
+					Console.WriteLine (e);
+				}
+				return null;
+#else
 				try {
 					Exif.ExifContent exif_content = this.ExifData.GetContents (Exif.Ifd.Exif);
 					Exif.ExifEntry entry = exif_content.Lookup (Exif.Tag.UserComment);
@@ -69,6 +86,7 @@ namespace FSpot {
 					// errors here shouldn't be fatal
 					return null;
 				}
+#endif				
 			}
 		}
 
@@ -184,7 +202,7 @@ namespace FSpot {
 		public Exif.ExifData ExifData {
 			get {
 				if (exif_data == null) {
-					exif_data = new Exif.ExifData (path);
+					exif_data = Header.Exif;
 
 					if (exif_data.Handle.Handle == System.IntPtr.Zero)
 						exif_data = new Exif.ExifData ();
@@ -200,12 +218,12 @@ namespace FSpot {
 		public override PixbufOrientation GetOrientation () 
 		{
 			PixbufOrientation orientation = PixbufOrientation.TopLeft;
-#if true
+#if USE_TIFF
 			try {
 				DirectoryEntry e = ExifHeader.Directory.Lookup (TagId.Orientation);
 				orientation = (PixbufOrientation)e.ValueAsLong [0];
 			} catch {
-
+				System.Console.WriteLine ("error checking orientation");
 			}
 #else						     
 			Exif.ExifEntry e = this.ExifData.GetContents (Exif.Ifd.Zero).Lookup (Exif.Tag.Orientation);
@@ -239,35 +257,32 @@ namespace FSpot {
 			get {
 				System.DateTime time;
 				try {
-#if true
-					using (Exif.ExifData ed = new Exif.ExifData (path)) {
-						string time_str = "";				
-						time_str = ed.LookupFirstValue (Exif.Tag.DateTimeOriginal);
-
-						if (time_str == null || time_str == "") 
-							time_str = ed.LookupFirstValue (Exif.Tag.DateTime);
-
-						time = Exif.ExifUtil.DateTimeFromString (time_str).ToUniversalTime (); 
-#else
-						TiffHeader tiff = Header.GetExifHeader ();
-						SubdirectoryEntry sub = (SubdirectoryEntry) tiff.Directory.Lookup (TagId.ExifIfdPointer);
-						DirectoryEntry e;
+#if USE_TIFF
+					SubdirectoryEntry sub = (SubdirectoryEntry) ExifHeader.Directory.Lookup (TagId.ExifIfdPointer);
+					DirectoryEntry e;
+					
+					if (sub != null) {
+						e = sub.Directory [0].Lookup (TagId.DateTimeOriginal);
 						
-						if (sub != null) {
-							e = sub.Directory [0].Lookup (TagId.DateTimeOriginal);
-							
-							if (e != null)
-								return DirectoryEntry.DateTimeFromString (e.StringValue).ToUniversalTime ();
-						}
-						
-						e = tiff.Directory.Lookup (TagId.DateTime);
-
 						if (e != null)
 							return DirectoryEntry.DateTimeFromString (e.StringValue).ToUniversalTime ();
-						
-						return base.Date;
-#endif
 					}
+					
+					e = ExifHeader.Directory.Lookup (TagId.DateTime);
+					
+					if (e != null)
+						return DirectoryEntry.DateTimeFromString (e.StringValue).ToUniversalTime ();
+					
+					return base.Date;
+#else
+					string time_str = "";				
+					time_str = ExifData.LookupFirstValue (Exif.Tag.DateTimeOriginal);
+					
+					if (time_str == null || time_str == "") 
+						time_str = ExifData.LookupFirstValue (Exif.Tag.DateTime);
+					
+					time = Exif.ExifUtil.DateTimeFromString (time_str).ToUniversalTime (); 
+#endif
 				} catch (System.Exception e) {
 					time = base.Date;
 				}
