@@ -59,7 +59,6 @@ public class PixbufLoader {
 	   thread that there are pending items in the
 	   `processed_requests' queue.  */
 	ThreadNotify pending_notify;
-
 	/* Whether a notification is pending on `pending_notify'
 	   already or not.  */
 	private bool pending_notify_notified;
@@ -119,6 +118,16 @@ public class PixbufLoader {
 		}
 	}
 
+	public void Request (Uri uri, int order)
+	{
+		Request (uri.ToString (), order);
+	}
+
+	public void Request (Uri uri, int order, int width, int height)
+	{
+		Request (uri.ToString (), order, width, height);
+	}
+
 	public void Cancel (string path)
 	{
 		lock (queue) {
@@ -136,12 +145,20 @@ public class PixbufLoader {
 	{
 		Pixbuf orig_image;
 		try {
-			if (request.width > 0) {
-				orig_image = PixbufUtils.LoadAtMaxSize (request.path, request.width, request.height);
-			} else
-				orig_image = new Pixbuf (request.path);
+			FSpot.ImageFile img;
 
-		} catch (GLib.GException ex){
+			if (System.IO.File.Exists (request.path))
+				img = FSpot.ImageFile.Create (request.path);
+		        else
+				img = FSpot.ImageFile.Create (new Uri (request.path));
+
+			if (request.width > 0) {
+				orig_image = img.Load (request.width, request.height);
+			} else
+				orig_image = img.Load ();
+
+		} catch (GLib.GException e){
+			System.Console.WriteLine (e.ToString ());
 			return;		
 		}
 		
