@@ -7,6 +7,57 @@ using GLib;
 using System.Runtime.InteropServices;
 
 namespace FSpot {
+	public class XScreenSaverSlide : Gtk.Window {
+		public const string ScreenSaverEnviroment = "XSCREENSAVER_WINDOW";
+
+		public XScreenSaverSlide () : base ("")
+		{
+		}
+	       
+		protected override void OnRealized ()
+		{
+			string env = Environment.GetEnvironmentVariable (ScreenSaverEnviroment);
+			
+			if (env != null) {
+				try {
+					env = env.ToLower ();
+					
+					if (env.StartsWith ("0x"))
+						env = env.Substring (2);
+
+					uint xid = UInt32.Parse (env, System.Globalization.NumberStyles.HexNumber);
+					
+					GdkWindow = Gdk.Window.ForeignNew (xid);
+					Style.Attach (GdkWindow);
+					GdkWindow.Events = EventMask.ExposureMask 
+						| EventMask.StructureMask 
+						| EventMask.EnterNotifyMask 
+						| EventMask.LeaveNotifyMask 
+						| EventMask.FocusChangeMask;
+					
+					Style.SetBackground (GdkWindow, Gtk.StateType.Normal);
+					GdkWindow.SetDecorations ((Gdk.WMDecoration) 0);
+					GdkWindow.UserData = this.Handle;
+					SetFlag (WidgetFlags.Realized);
+					SizeRequest ();
+					Gdk.Rectangle geom;
+					int depth;
+					GdkWindow.GetGeometry (out geom.X, out geom.Y, out geom.Width, out geom.Height, out depth);
+					SizeAllocate (new Gdk.Rectangle (geom.X, geom.Y, geom.Width, geom.Height));
+					Resize (geom.Width, geom.Height);
+					return;
+				} catch (System.Exception e) {
+					System.Console.WriteLine (e);
+				}
+			} else {
+				System.Console.WriteLine ("{0} not set, falling back to window", ScreenSaverEnviroment);
+			}
+
+			SetSizeRequest (640, 480);
+			base.OnRealized ();
+		}
+	}
+
 	public class FullSlide : Gtk.Window {
 		private SlideView slideview;
 		private Gdk.Pixbuf screenshot;
@@ -38,9 +89,6 @@ namespace FSpot {
 			hide = new Delay (2000, new GLib.IdleHandler (HideCursor));
 		}
 
-		[DllImport("libgdk-2.0-0.dll")]
-		static extern IntPtr gdk_cursor_new_from_pixbuf (IntPtr display, IntPtr pixbuf, int x, int y);
-
 		public Gdk.Cursor Empty () 
 		{
 			Gdk.Cursor cempty = null;
@@ -48,8 +96,7 @@ namespace FSpot {
 			try {
 				Gdk.Pixbuf empty = new Gdk.Pixbuf (Gdk.Colorspace.Rgb, true, 8, 1, 1);
 			        empty.Fill (0x00000000);
-				IntPtr raw = gdk_cursor_new_from_pixbuf (this.GdkWindow.Display.Handle, empty.Handle, 0, 0);
-				cempty = new Gdk.Cursor (raw);
+				cempty = new Gdk.Cursor (GdkWindow.Display, empty, 0, 0);
 			} catch (System.Exception e){
 				System.Console.WriteLine (e.ToString ());
 				return null;
@@ -126,8 +173,9 @@ namespace FSpot {
 		uint flip_timer = 0;
 		uint transition_timer = 0;
 		
+		uint fail_count = 0;
 		bool animate = true;
-		uint animate_max = 160;
+		uint animate_max = 200;
 		
 		bool black = false;
 		uint flip_interval = 2000;
@@ -140,16 +188,15 @@ namespace FSpot {
 		}
 		
 		public bool Animate {
-			get {
-				return animate;
-			}
-			set {
-				animate = value;
-			}
+			get { return animate; }
+			set { animate = value; }
 		}
 		
 		public void Play () 
 		{
+			if (photos.Length < 1)
+				return;
+
 			StopTweenIdle ();
 			if (current_idx >= 0) {
 				Pixbuf frame = GetScaled (photos[current_idx]);
@@ -214,10 +261,11 @@ namespace FSpot {
 					next_idx = idx;
 					StartTweenIdle ();
 					
-
 					return true;
 				} else {
-					next.Dispose ();
+					if (next != null)
+						next.Dispose ();
+
 					next = GetScaled (photos [0]);
 					next_idx = 0;
 					StartTweenIdle ();
@@ -233,12 +281,24 @@ namespace FSpot {
 		
 		private Pixbuf CrossFade (Pixbuf current, Pixbuf prev, Pixbuf next, double percent)
 		{ 
-			int width = Allocation.Width;
-			int height = Allocation.Height;
-			
-			prev.CopyArea (0, 0, width, height, current, 0, 0);
-			next.Composite (current, 0,0, width, height, 0, 0, 1, 1,
+			Rectangle area = new Rectangle (0, 0, Allocation.Width, Allocation.Height);
+			Rectangle rect = new Rectangle (0, 0, 256, 256);
+			Rectangle subarea;
+
+			while (rect.Y < area.Height) {
+				while (rect.X < area.Width) {
+					if (IsRealized)
+						GdkWindow.ProcessUpdates (false);
+
+					rect.Intersect (area, out subarea);
+					prev.CopyArea (subarea.X, subarea.Y, subarea.Width, subarea.Height, current, subarea.X, subarea.Y);
+					next.Composite (current, subarea.X, subarea.Y, subarea.Width, subarea.Height, 0, 0, 1, 1,
 					Gdk.InterpType.Nearest, (int) System.Math.Round (255 * percent));
+					rect.X += rect.Width;
+				}
+				rect.X = area.X;
+				rect.Y += rect.Height;
+			}
 			return current;
 		}
 		
@@ -269,7 +329,35 @@ namespace FSpot {
 		
 		private Pixbuf GetScaled (Pixbuf orig)
 		{
-			Pixbuf scaled = PixbufUtils.ScaleToAspect (orig, Allocation.Width, Allocation.Height);
+			Gdk.Rectangle pos;
+			int width = Allocation.Width;
+			int height = Allocation.Height;
+			double scale = PixbufUtils.Fit (orig, width, height, false, out pos.Width, out pos.Height);
+			pos.X = (width - pos.Width) / 2;
+			pos.Y = (height - pos.Height) / 2;
+			
+			Pixbuf scaled = new Pixbuf (Colorspace.Rgb, false, 8, width, height);
+			scaled.Fill (0x000000); 
+			
+			Rectangle rect = new Rectangle (pos.X, pos.Y, 256, 256);
+			Rectangle subarea;
+			
+			while (rect.Top < pos.Bottom) {
+				while (rect.X < pos.Right) {
+					if (IsRealized) 
+						GdkWindow.ProcessUpdates (false);
+
+					rect.Intersect (pos, out subarea);
+					orig.Composite (scaled, subarea.X, subarea.Y, 
+							subarea.Width, subarea.Height,
+							pos.X, pos.Y, scale, scale,
+							Gdk.InterpType.Bilinear,
+							255);
+					rect.X += rect.Width;
+				}
+				rect.X = pos.X;
+				rect.Y += rect.Height;
+			}
 			
 			orig.Dispose ();
 			return scaled;
@@ -277,7 +365,13 @@ namespace FSpot {
 		
 		private Pixbuf GetScaled (IBrowsableItem photo)
 		{
-			Pixbuf orig = FSpot.PhotoLoader.LoadAtMaxSize (photo, Allocation.Width, Allocation.Height);
+			Pixbuf orig;
+			try { 
+				orig = FSpot.PhotoLoader.LoadAtMaxSize (photo, Allocation.Width, Allocation.Height);
+			} catch {
+				orig = null;
+			}
+
 			if (orig == null)
 				return null;
 
@@ -300,23 +394,31 @@ namespace FSpot {
 		
 		private bool HandleTransitionTimer ()
 		{			
+			System.DateTime start_time = System.DateTime.Now;
 			transition_timer = 0;
 			if (current_tween--  > 0) {
 				StartTransitionTimer ();
-				System.DateTime start_time = System.DateTime.Now;
 				this.Pixbuf = tweens[current_tween];
 				GdkWindow.ProcessUpdates (false);
 				System.TimeSpan span = System.DateTime.Now  - start_time;
 				
-				if (animate && span.TotalMilliseconds > animate_max) {
-					animate = false;
-					System.Console.WriteLine ("Disabling slide animation due to excessive frame interval {0}ms", 
-								  span.TotalMilliseconds);
-					current_tween = 0;
-				}
+				if (Animate) { 
+					if (span.TotalMilliseconds > animate_max) {
+						fail_count++;
+						
+						if (fail_count > 3) {
+							Animate = false;
+							System.Console.WriteLine ("Disabling slide animation due to 3 consecutive excessive frame intervals {0}ms", 
+										  span.TotalMilliseconds);
+							current_tween = 0;
+						}
+					} else {
+						fail_count = 0;
+					}
+				} 
 			} else {
 				ShowNext ();
-				
+
 				PreloadNextImage (current_idx + 1);
 				StartFlipTimer ();
 			}
@@ -328,7 +430,12 @@ namespace FSpot {
 		private bool HandleTweenIdle ()
 		{
 			using (Pixbuf prev = this.Pixbuf) {	
-				if (!animate) {
+				if (!Animate) {
+					ClearTweens ();
+					return false;
+				}
+				
+				if (photos.Length < 2) { // Only one photo. Nothing to do
 					ClearTweens ();
 					return false;
 				}
@@ -383,9 +490,9 @@ namespace FSpot {
 		
 		private void StopTranstionTimer ()
 		{
-			if (transition_timer != 0) {
+			if (transition_timer != 0)
 				GLib.Source.Remove (transition_timer);
-			}
+
 			transition_timer = 0;
 		}
 		
@@ -398,30 +505,31 @@ namespace FSpot {
 		
 		private void StopFlipTimer ()
 		{	
-			if (flip_timer != 0) {
+			if (flip_timer != 0)
 				GLib.Source.Remove (flip_timer);
-			}
+
 			flip_timer = 0;
 		}
 
 		
 		private void HandleSizeAllocate (object sender, SizeAllocatedArgs args)
 		{	
-			if (Pixbuf == null)
+			Pixbuf current = this.Pixbuf;
+
+			if (current == null)
 				return;
-			
+
 			//
 			// The size has changed so we need to reload the images.
 			//
-			Pixbuf current = this.Pixbuf;
 			if (current.Width != Allocation.Width || current.Height != Allocation.Height) {
 				bool playing = (flip_timer != 0 || transition_timer != 0);
 				
 				if (current_idx < 0) {
-					Gdk.Pixbuf old = this.Pixbuf;
-					this.Pixbuf = GetScaled (this.Pixbuf);
-					if (old != this.Pixbuf)
-						old.Dispose ();
+					using (Gdk.Pixbuf old = this.Pixbuf) {
+						this.Pixbuf = GetScaled (old);
+						current.Dispose ();
+					}
 				} else {
 					using (Pixbuf frame =  GetScaled (photos[current_idx])) {
 						this.Pixbuf =  frame;
@@ -442,9 +550,9 @@ namespace FSpot {
 
 		private void ClearTweens () {
 			for (int i = 0; i < tweens.Length; i++) {
-			if (tweens[i] != null) 
-				tweens[i].Dispose ();
-			tweens[i] = null;
+				if (tweens[i] != null) 
+					tweens[i].Dispose ();
+				tweens[i] = null;
 			}
 		}
 		
