@@ -373,6 +373,7 @@ namespace FSpot.Tiff {
 				enc = System.Text.Encoding.ASCII;
 				break;
 			case "UNICODE\0":
+			case "Unicode\0":
 				enc = new System.Text.UnicodeEncoding (! little, true);
 				break;
 			case "SJIS\0\0\0\0":
@@ -416,7 +417,7 @@ namespace FSpot.Tiff {
 				heading = "ASCII\0\0\0";
 				enc = new System.Text.ASCIIEncoding ();
 			} else {
-				heading = "UNICODE\0";
+				heading = "Unicode\0";
 				enc = new System.Text.UnicodeEncoding (! is_little, true);
 			}
 			
@@ -1832,12 +1833,27 @@ namespace FSpot.Tiff {
 		public TiffFile (string path) : base (path)
 		{
 			try {
-				using (System.IO.Stream input = System.IO.File.OpenRead (path)) {
+				using (System.IO.Stream input = Open ()) {
 					this.Header = new Header (input);
 				}
 
 #if DEBUG_LOADER
-				Header.Dump (path + ":");
+				Header.Dump (this.ToString () + ":");
+#endif
+			} catch (System.Exception e) {
+				System.Console.WriteLine (e.ToString ());
+			}
+		}
+
+		public TiffFile (Uri uri) : base (uri)
+		{
+			try {
+				using (System.IO.Stream input = Open ()) {
+					this.Header = new Header (input);
+				}
+
+#if DEBUG_LOADER
+				Header.Dump (this.ToString () + ":");
 #endif
 			} catch (System.Exception e) {
 				System.Console.WriteLine (e.ToString ());
@@ -1872,7 +1888,7 @@ namespace FSpot.Tiff {
 		
 		public override System.IO.Stream PixbufStream ()
 		{
-			return null;
+			return Open ();
 		}
 
 		public override PixbufOrientation GetOrientation ()
@@ -1888,7 +1904,7 @@ namespace FSpot.Tiff {
 		{
 			uint offset = directory.Lookup (TagId.JPEGInterchangeFormat).ValueAsLong [0];
 			
-			System.IO.Stream file = System.IO.File.OpenRead (this.path);
+			System.IO.Stream file = Open ();
 			file.Position = offset;
 			return file;
 		}
@@ -1898,7 +1914,7 @@ namespace FSpot.Tiff {
 			uint offset = directory.Lookup (TagId.JPEGInterchangeFormat).ValueAsLong [0];
 			uint length = directory.Lookup (TagId.JPEGInterchangeFormatLength).ValueAsLong [0];
 			   
-			using (System.IO.Stream file = System.IO.File.OpenRead (this.path)) {
+			using (System.IO.Stream file = Open ()) {
 				file.Position = offset;
 				
 				byte [] data = new byte [32768];
@@ -1926,6 +1942,10 @@ namespace FSpot.Tiff {
 		{
 		}
 
+		public DngFile (System.Uri uri) : base (uri) 
+		{
+		}
+
 		public override System.IO.Stream PixbufStream ()
 		{
 			try {
@@ -1933,19 +1953,14 @@ namespace FSpot.Tiff {
 				ImageDirectory directory = sub.Directory [sub.Directory.Length - 1];
 
 				uint offset = directory.Lookup (TagId.StripOffsets).ValueAsLong [0];
-				System.IO.Stream file = System.IO.File.OpenRead (this.path);
+				System.IO.Stream file = Open ();
 				file.Position = offset;
 				return file;
 			} catch {
-				return DCRawFile.RawPixbufStream (path);
+				return DCRawFile.RawPixbufStream (uri);
 			}
 		}
 
-		public override Gdk.Pixbuf Load ()
-		{
-			return DCRawFile.Load (this.path, null);
-		}
-
 		public override void Select (SemWeb.StatementSink sink)
 		{
 			
@@ -2008,6 +2023,10 @@ namespace FSpot.Tiff {
 		{
 		}
 
+		public NefFile (Uri uri) : base (uri)
+		{
+		}
+
 		public override void Select (SemWeb.StatementSink sink)
 		{
 			DirectoryEntry e = Header.Directory.Lookup (TagId.NewSubfileType);
@@ -2044,7 +2063,9 @@ namespace FSpot.Tiff {
 
 		public Gdk.Pixbuf GetEmbeddedThumbnail ()
 		{
-			return TransformAndDispose (new Gdk.Pixbuf (path));
+			using (System.IO.Stream stream = Open ()) {
+				return TransformAndDispose (new Gdk.Pixbuf (stream));
+			}
 		}
 
 		public override System.IO.Stream PixbufStream ()
@@ -2054,39 +2075,21 @@ namespace FSpot.Tiff {
 				ImageDirectory jpeg_directory = sub.Directory [0];
 				return LookupJpegSubstream (jpeg_directory);
 			} catch (System.Exception e) {
-				return DCRawFile.RawPixbufStream (path);
+				return DCRawFile.RawPixbufStream (uri);
 			}
 		}
-		
-		public override Gdk.Pixbuf Load () 
-		{
-			Gdk.Pixbuf pixbuf = null;
-			System.Console.WriteLine ("starting load");
-			
-			try {
-				SubdirectoryEntry sub = (SubdirectoryEntry) Header.Directory.Lookup (TagId.SubIFDs);
-				ImageDirectory jpeg_directory = sub.Directory [0];
-				
-				pixbuf = LoadJpegInterchangeFormat (jpeg_directory);
-			} catch (System.Exception e) {
-				System.Console.WriteLine (e);
-				pixbuf = null;
-			}
-
-			if (pixbuf == null)
-				return DCRawFile.Load (this.Path, null);
-			
-			return TransformAndDispose (pixbuf);
-		}
 	}
 		
 
 	public class Cr2File : TiffFile, IThumbnailContainer {
-
 		public Cr2File (string path) : base (path) 
 		{
 		}
 
+		public Cr2File (Uri uri) : base (uri)
+		{
+		}
+
 		/*
 		public override PixbufOrientation GetOrientation ()
 		{
@@ -2105,16 +2108,9 @@ namespace FSpot.Tiff {
 		public override System.IO.Stream PixbufStream ()
 		{
 			uint offset = Header.Directory.Lookup (TagId.StripOffsets).ValueAsLong [0];
-			System.IO.Stream file = System.IO.File.OpenRead (this.path);
+			System.IO.Stream file = Open ();
 			file.Position = offset;
 			return file;
-			//return LookupJpegSubstream (Header.Directory.NextDirectory);
-			//return DCRawFile.RawPixbufStream (path);
-		}
-
-		public override Gdk.Pixbuf Load ()
-		{
-			return DCRawFile.Load (this.Path, null);
 		}
 	}
 }
